/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.java.net
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.nodes;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.List;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Node;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.netbeans.nodes.role.ChildFactoryExceptionHandler;
import it.tidalwave.util.logging.Logger;

/***********************************************************************************************************************
 *
 * This interface models the semantics of filtering a certain object.
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public abstract class SafeChildFactory<T> extends ChildFactory<T>
  {
    private static final String CLASS = SafeChildFactory.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    @Nonnull
    private Node node;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void setNode (final @Nonnull Node node)
      {
        this.node = node;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void retry()
      {
        if (node != null)
          {
            node.getChildren().add(new Node[] {createWaitNode()});
          }

        super.refresh(false);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected abstract boolean createKeysSafely (@Nonnull List<T> list)
      throws Exception;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    protected final boolean createKeys (final @Nonnull List<T> list)
      {
        try
          {
            return createKeysSafely(list);
          }
        catch (Exception e)
          {
            ChildFactoryExceptionHandler nodeExceptionHandler = null;

            try
              {
                nodeExceptionHandler = findExceptionHandler(node);
              }
            catch (NotFoundException e2)
              {
                logger.finest(">>>> no specific NodeExceptionHandler, using the default");
                nodeExceptionHandler = Locator.find(ChildFactoryExceptionHandler.class);
              }

            nodeExceptionHandler.handleException(node, this, e);
            return true;
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    private ChildFactoryExceptionHandler findExceptionHandler (final @CheckForNull Node node)
      throws NotFoundException
      {
        logger.finest("findNodeExceptionHandler(%s)", node);
        
        NotFoundException.throwWhenNull(node, "");
        final ChildFactoryExceptionHandler exceptionHandler = node.getLookup().lookup(ChildFactoryExceptionHandler.class);

        return (exceptionHandler != null) ? exceptionHandler : findExceptionHandler(node.getParentNode());
      }
  }
