/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.java.net
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.nodes.role;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import lombok.Getter;
import lombok.Setter;
import org.openide.util.NbBundle;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Node;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.netbeans.nodes.SafeChildFactory;
import it.tidalwave.netbeans.nodes.role.impl.ErrorNode;
import it.tidalwave.netbeans.nodes.role.impl.Retrier;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class DefaultChildFactoryExceptionHandler implements ChildFactoryExceptionHandler
  {
    private static final String CLASS = DefaultChildFactoryExceptionHandler.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    @Getter @Setter @Nonnull
    private String errorMessage =
            NbBundle.getMessage(DefaultChildFactoryExceptionHandler.class, "ChildFactoryExceptionHandler.error");

    @Getter @Setter @Nonnull
    private String errorIconPath =
            NbBundle.getMessage(DefaultChildFactoryExceptionHandler.class, "ChildFactoryExceptionHandler.errorIcon");

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void handleException (final @CheckForNull Node node,
                                 final @Nonnull ChildFactory<?> childFactory,
                                 final @Nonnull Throwable throwable)
      {
        logger.fine("handleException(%s, %s, %s)", node, childFactory, throwable);
        logger.warning("While populating children nodes: ", throwable);
        logger.throwing(CLASS, "handleException()", throwable);
        
        if (node != null)
          {
            final Retrier retrier = new Retrier(node.getChildren(), (SafeChildFactory<?>) childFactory);
            node.getChildren().add(new Node[]{new ErrorNode(errorMessage, errorIconPath, throwable, retrier)});
          }
      }
  }
