/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.java.net
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.nodes.role;

import javax.annotation.Nonnull;
import javax.annotation.PostConstruct;
import javax.annotation.concurrent.ThreadSafe;
import javax.inject.Inject;
import org.openide.nodes.Node;
import org.openide.util.Lookup;
import org.openide.util.Lookup.Result;
import org.openide.util.LookupEvent;
import org.openide.util.LookupListener;
import it.tidalwave.netbeans.role.DataLoader;
import it.tidalwave.util.NotFoundException;

/***********************************************************************************************************************
 *
 * A specification of {@link DataLoader} working with {@link Node}s. It feeds the retrieved {@code Node} to a
 * {@link NodeManager} by means of a {@link RootNodeProvider}. It eventually decorates the {@code Node} if one or more
 * {@link NodeDecorator}s are found in the context.
 *
 * @stereotype Role
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@ThreadSafe
public abstract class NodeDataLoader implements DataLoader
  {
    @Inject
    private NodeManager nodeManager;

    @Inject
    private Lookup lookup;

    @Nonnull // TODO Would it be nice to make a Result<?> directly injectable
    private Result<NodeDecorator> nodeDecorators;

    /*******************************************************************************************************************
     *
     *
     *
     ******************************************************************************************************************/
    @Override
    public final void loadData()
      {
        nodeManager.setRootNodeProvider(new RootNodeProvider()
          {
            @Override
            @Nonnull
            public Node getRootNode()
              throws NotFoundException
              {
                Node node = NodeDataLoader.this.getRootNode();

                for (final NodeDecorator nodeDecorator : nodeDecorators.allInstances())
                  {
                    node = nodeDecorator.decorate(node);
                  }

                return node;
              }
          });
      }

    /*******************************************************************************************************************
     *
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected abstract Node getRootNode()
      throws NotFoundException;

    /*******************************************************************************************************************
     *
     *
     *
     ******************************************************************************************************************/
    @PostConstruct
    @SuppressWarnings("PMD.UnusedPrivateMethod") @edu.umd.cs.findbugs.annotations.SuppressWarnings("UPM_UNCALLED_PRIVATE_METHOD")
    private void initialize()
      {
        nodeDecorators = lookup.lookupResult(NodeDecorator.class);
        nodeDecorators.addLookupListener(new LookupListener()
          {
            @Override
            public void resultChanged (final @Nonnull LookupEvent event)
              {
                loadData();
              }
          });
      }
  }
