/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.java.net
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.nodes.role.impl;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import javax.annotation.Nonnull;
import lombok.Getter;
import it.tidalwave.util.Finder;
import it.tidalwave.util.Finder.SortCriterion;
import it.tidalwave.util.Finder.SortDirection;
import it.tidalwave.role.Sortable;

/***********************************************************************************************************************
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class DefaultSortable implements Sortable
  {
    public static final String PROP_SORT_DIRECTION = "sortDirection";

    public static final String PROP_SORT_CRITERION = "sortCriterion";

    @Nonnull @Getter
    private Finder.SortDirection sortDirection = Finder.SortDirection.ASCENDING;

    @Nonnull @Getter
    private Finder.SortCriterion sortCriterion = Finder.SortCriterion.UNSORTED;
    
    private final PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);

    @Override
    public void setSortCriterion (final @Nonnull SortCriterion sortCriterion) 
      {
        final SortCriterion oldSortCriterion = this.sortCriterion;
        this.sortCriterion = sortCriterion;
        propertyChangeSupport.firePropertyChange(PROP_SORT_CRITERION, oldSortCriterion, sortCriterion);
      }

    @Override
    public void setSortDirection (final @Nonnull SortDirection sortDirection) 
      {
        final SortDirection oldSortDirection = this.sortDirection;
        this.sortDirection = sortDirection;
        propertyChangeSupport.firePropertyChange(PROP_SORT_DIRECTION, oldSortDirection, sortDirection);
      }
    
    public void addPropertyChangeListener (final @Nonnull PropertyChangeListener listener) 
      {
        propertyChangeSupport.addPropertyChangeListener(listener);
      }

    public void removePropertyChangeListener (final @Nonnull PropertyChangeListener listener) 
      {
        propertyChangeSupport.removePropertyChangeListener(listener);
      }
  }
