/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.java.net
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.nodes.role.impl;

import java.util.Arrays;
import javax.annotation.Nonnull;
import org.openide.nodes.Node;

/*********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class ReflectiveNodeFactory 
  {
    @Nonnull
    private final Object[] extraArgs;

    @Nonnull
    private final Class<?>[] argTypes;
    
    public ReflectiveNodeFactory  (final @Nonnull Object[] extraArgs)
      {
        this.extraArgs = extraArgs;

        argTypes = new Class<?>[extraArgs.length + 1];
        argTypes[0] = Node.class;

        // TODO: should be more tolerant and accept any supertype...
        for (int i = 0; i < extraArgs.length; i++)
          {
            argTypes[i + 1] = extraArgs[i].getClass().getInterfaces()[0];
          }
      }
    
    @Nonnull
    public Node createNode (final @Nonnull Class<? extends Node> nodeClass,
                            final @Nonnull Node delegate)
      {
        final Object[] args = new Object[extraArgs.length + 1];
        args[0] = delegate;
        System.arraycopy(extraArgs, 0, args, 1, extraArgs.length);

        try
          {
            return nodeClass.getConstructor(argTypes).newInstance(args);
          }
        catch (NoSuchMethodException e) 
          {
            throw newExceptionBadConstructor(nodeClass, e);
          }
        catch (SecurityException e) 
          {
            throw newExceptionBadConstructor(nodeClass, e);
          }
        catch (Exception e)
          {
            // RuntimeError would be caught by the Platform runtime
            throw new Error(e);
          }
      }

    @Nonnull
    private Error newExceptionBadConstructor (final @Nonnull Class<? extends Node> parentNodeClass, 
                                              final @Nonnull Exception cause) 
      {
        // RuntimeError would be caught by the Platform runtime
        return new Error(parentNodeClass + " must have a public constructor accepting " +
                         Arrays.toString(argTypes), cause);
      }
  }
