/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.java.net
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.nodes.role.impl;

import java.lang.reflect.InvocationTargetException;
import javax.annotation.Nonnull;
import java.awt.EventQueue;
import org.openide.nodes.Children;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.netbeans.nodes.SafeChildFactory;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class Retrier 
  {
    private static final String CLASS = Retrier.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    @Nonnull
    private final Children children;

    @Nonnull
    private final SafeChildFactory<?> childFactory;

    public Retrier (final @Nonnull Children children, final @Nonnull SafeChildFactory<?> childFactory)
      {
        this.children = children;
        this.childFactory = childFactory;
      }

    public void retry()
      {
        try
          {
            EventQueue.invokeAndWait(new Runnable()
              {
                @Override
                public void run()
                  {
                    logger.finest(">>>> removing current children...");
                    children.remove(children.getNodes());
                  }
              });

            logger.finer(">>>> retrying creating children...");
            childFactory.retry();
          }
        catch (InvocationTargetException e)
          {
          }
        catch (InterruptedException e)
          {
          }
      }
  }
